//---------------------------------------------------------------------
#include "MainWindow.h"
//---------------------------------------------------------------------
BAppMainWindow::BAppMainWindow(BRect frame,const char *title)
    :BWindow(frame,title,MAINWINDOW_WINDOWSTYLE,0)
{
    mainview=new BAppMainView(Bounds());	
    AddChild(mainview);

    nsock=-1;
    vtcp=NULL;
}
//---------------------------------------------------------------------
void BAppMainWindow::MessageReceived(BMessage *msg)
{
    switch(msg->what)
    {
        case MSG_SEND:
            sock_send();
            break;
        case MSG_CONNECT:
            sock_connect();
            break;
        case MSG_LISTEN:
            sock_listen();
            break;
        case MSG_VTCP_CLOSED:
            vtcp->sock_closed();
       		mainview->strview->SetText("Disconnect");	
            break;
        case MSG_VTCP_RECV:
            {
                char *buf;
                ssize_t leng;
				
                msg->FindData("data",B_ANY_TYPE,(const void **)&buf,&leng);

                mainview->strview->SetText(buf);
            }
            break;
        default:
            BWindow::MessageReceived(msg);
    }
}
//---------------------------------------------------------------------
BAppMainView::BAppMainView(BRect frame)
    :BView(frame,"bappmainview",B_FOLLOW_ALL,B_WILL_DRAW)
{
    BRect viewrect(Bounds());
    connectbtn=new BButton(BRect(viewrect.right-144,8,
                                 viewrect.right-80,28),"connectbtn",
                                 "Connect",new BMessage(MSG_CONNECT),
	                             B_FOLLOW_RIGHT | B_FOLLOW_TOP);
    AddChild(connectbtn);
    listenbtn=new BButton(BRect(viewrect.right-72,8,
                                viewrect.right-8,28),"listenbtn",
                                "Listen",new BMessage(MSG_LISTEN),
                                B_FOLLOW_RIGHT | B_FOLLOW_TOP);
    AddChild(listenbtn);
    sendbtn=new BButton(BRect(viewrect.right-72,98,
                              viewrect.right-8,118),"sendbtn",
                              "Send",new BMessage(MSG_SEND),
                              B_FOLLOW_RIGHT | B_FOLLOW_TOP);
    AddChild(sendbtn);

    textctrl=new BTextControl(BRect(8,38,viewrect.right-8,58),"textctrl",
                              NULL,"Default",NULL,
                              B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
    AddChild(textctrl);

    strview=new BStringView(BRect(8,68,viewrect.right-8,88),"strview",
                            "NoData",
                            B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
    AddChild(strview);

	portsw=new BCheckBox(BRect(8,98,40,118),"portsw","On:A,Off:B",NULL);
    AddChild(portsw);

    textctrl->SetEnabled(false);
    sendbtn->SetEnabled(false);
}
//---------------------------------------------------------------------
void BAppMainWindow::sock_connect(void)
{
    mainview->connectbtn->SetEnabled(false);
    mainview->listenbtn->SetEnabled(false);

	vtcp=new VTcpNetworkClient(this);
	
	nsock=vtcp->sock_connect("127.0.0.1",
	            10000+((mainview->portsw->Value()==B_CONTROL_ON)?1:0));

    if(nsock!=-1)
    {
		mainview->strview->SetText("Connect");	
		mainview->textctrl->SetEnabled(true);
		mainview->sendbtn->SetEnabled(true);
	}
}
//---------------------------------------------------------------------
void BAppMainWindow::sock_listen(void)
{
    int ssock;
	struct sockaddr_in sa;
	int sa_size;
    
	mainview->connectbtn->SetEnabled(false);
	mainview->listenbtn->SetEnabled(false);

	ssock=socket(AF_INET,SOCK_STREAM,0);

    sa.sin_family=AF_INET;
    sa.sin_port=htons(
                10000+((mainview->portsw->Value()==B_CONTROL_ON)?1:0));
    sa.sin_addr.s_addr=INADDR_ANY;
    memset(sa.sin_zero,0,sizeof(sa.sin_zero));
    bind(ssock,(struct sockaddr *)&sa,sizeof(sa)); 
	listen(ssock,5);
	sa_size=sizeof(sa);

	vtcp=new VTcpNetworkClient(this);
	nsock=vtcp->sock_accept(ssock); 
    closesocket(ssock);

	if(nsock!=-1)
	{
		mainview->strview->SetText("Connect");	
		mainview->textctrl->SetEnabled(true);
		mainview->sendbtn->SetEnabled(true);
	}
}
//---------------------------------------------------------------------
void BAppMainWindow::sock_send(void)
{
	int r;
	
	r=vtcp->sock_send((void *)mainview->textctrl->Text(),
	     (int32)strlen(mainview->textctrl->Text())+1);
	if(r==-1)
	{
		nsock=-1;
		mainview->strview->SetText("Disconnect");
	}
}
//---------------------------------------------------------------------
bool BAppMainWindow::QuitRequested()
{
	if(vtcp!=NULL)
		vtcp->sock_closed();
	
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
};
//---------------------------------------------------------------------
